;+
;; Name:
;    noaa_load_kp
;
; Purpose:
;    Loads local Kp/Ap index data into tplot variables. If data doesn't exist locally, the code
;    downloads the data from the THEMIS mirror of NOAA STP's data
;
; Syntax:
;    noaa_load_kp, [, trange = trange]
;                 
; Keywords:
;    trange: time range to load
;    kp_mirror: http server where mirrored Kp/Ap data lives
;    remote_kp_dir: directory where the Kp/Ap data lives on the mirror server
;    local_kp_dir: directory where data lives locally
;
; $LastChangedBy: egrimes $
; $LastChangedDate: 2013-05-23 09:45:15 -0700 (Thu, 23 May 2013) $
; $LastChangedRevision: 12391 $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/ssl_general/trunk/missions/noaa/noaa_load_kp.pro $
;-
pro noaa_load_kp,  trange = trange, kp_mirror = kp_mirror, remote_kp_dir=remote_kp_dir, local_kp_dir=local_kp_dir
    if ~keyword_set(trange) then trange = timerange()
    if ~keyword_set(kp_mirror) then kp_mirror = 'http://themis-data.igpp.ucla.edu/'
    if ~keyword_set(local_kp_dir) then file_prefix = root_data_dir() + 'geom_indices\' $
        else file_prefix = local_kp_dir
    if ~keyword_set(remote_kp_dir) then remote_kp_dir = 'thg/mirrors/kp/noaa/'
    
    starttime = time_struct(trange[0])
    endtime = time_struct(trange[1])
    
    years = starttime.year+indgen(endtime.year-starttime.year+1)
    nyears = n_elements(years)

    kpdata = intarr(8.*366.*nyears)
    apdata = kpdata
    kptimes = dblarr(8.*366.*nyears)
    daytimes = dblarr(366.*nyears)
    srndata = intarr(366.*nyears)
    srddata = srndata
    kpsdata = srndata
    apmdata = srndata
    cpdata = fltarr(366.*nyears)
    c9data = srndata
    ssndata = srndata
    srfdata = cpdata
    fqdata = srndata
    j = 0 ; counter
     
    ; loop through years
    for i=0, nyears-1 do begin
        kpfile = file_prefix+strcompress(years[i],/rem)
        
        ; try to open the Kp data file
        get_lun, lun
        openr, lun, kpfile, error=err
       
        ; file doesn't exist locally, download from a mirror
        if (!error_state.name eq 'IDL_M_CNTOPNFIL') then begin
            file = file_retrieve(remote_kp_dir+strcompress(string(years[i]),/rem), remote_data_dir=kp_mirror, local_data_dir=file_prefix, /ascii_mode)
            openr, lun, file
        endif
        
        ; loop through file
        ; see ftp://ftp.ngdc.noaa.gov/STP/GEOMAGNETIC_DATA/INDICES/KP_AP/kp_ap.fmt for a full description of the data files
        while not eof(lun) do begin
            full_line = ''
            readf,lun,full_line
            year = strmid(full_line,0,2)
            month = strmid(full_line,2,2)
            day = strmid(full_line,4,2)
            srndata[j] = fix(strmid(full_line,6,4))
            srddata[j] = fix(strmid(full_line,10,2))
            for k = 0,7 do kpdata[k+j*8] = fix(strmid(full_line,12+2*k,2))
            kpsdata[j] = fix(strmid(full_line,28,3))
            for k = 0,7 do apdata[k+j*8] = fix(strmid(full_line,31+3*k,3))
            apmdata[j] = fix(strmid(full_line,55,3))
            cpdata[j] = float(strmid(full_line,58,3))
            c9data[j] = fix(strmid(full_line,61,1))
            ssndata[j] = fix(strmid(full_line,62,3))
            srfdata[j] = float(strmid(full_line,65,5))
            fqdata[j] = fix(strmid(full_line,70,1))
            kptimes[j*8.:(j+1)*8.-1.] = time_double(year+'-'+month+'-'+$
                day+'/'+string(indgen(8)*3)+':00:00')
            daytimes[j] = time_double(year+'-'+month+'-'+day)
            j = j+1.
        endwhile
        close, lun
        free_lun, lun
    endfor

    ; store the data as tplot variables
    store_data,'Kp',data={x: kptimes[0:j*8-1], y: kpdata[0:j*8-1]}
    store_data,'ap',data={x: kptimes[0:j*8-1], y: apdata[0:j*8-1]}
    store_data,'Sol_Rot_Num',data={x: daytimes[0:j-1], y: srndata[0:j-1]}
    store_data,'Sol_Rot_Day',data={x: daytimes[0:j-1], y: srddata[0:j-1]}
    store_data,'Kp_Sum',data={x: daytimes[0:j-1], y: kpsdata[0:j-1]}
    store_data,'ap_Mean',data={x: daytimes[0:j-1], y: apmdata[0:j-1]}
    store_data,'Cp',data={x: daytimes[0:j-1], y: cpdata[0:j-1]}
    store_data,'C9',data={x: daytimes[0:j-1], y: c9data[0:j-1]}
    store_data,'Sunspot_Number',data={x: daytimes[0:j-1], y: ssndata[0:j-1]}
    store_data,'Solar_Radio_Flux',data={x: daytimes[0:j-1], y: srfdata[0:j-1]}
    store_data,'Flux_Qualifier',data={x: daytimes[0:j-1], y: fqdata[0:j-1]}
end